/* File: Complex.java
 *                             -- A  Java  class  for performing complex
 *                                number arithmetic to double precision.
 *
 * Copyright (c) 1997 - 2001, Alexander Anderson.
 *
 * This  program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published  by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be  useful,  but
 * WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
 * General Public License for more details.
 *
 * You  should  have  received  a copy of the GNU General Public License
 * along  with  this  program;  if  not,  write  to  the  Free  Software
 * Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 *    Alexandr Korlyukov
 *    korlyukov@grsu.grodno.by
 */

import ORG.netlib.math.complex.Complex;

/**
 *  Class for calculation of number of the happy tickets
 *  through complex numbers.
 *  For example,
 *  123 204 - happy ticket, because the sum 1, 2, 3
 *                      is equal to the sum 2, 0, 4
 *  123 254 - is not the happy ticket, because the sum 1, 2, 3
 *                             is not equal to the sum 2, 5, 4
 */
public class HappyTickets{

    public static final int notation  =  2;
    public static final int lengthTicket  =  6;
    public static final int c28  = (notation-1)*(lengthTicket/2)+1;

    public static int numberSteps  =  1000000;

    public static final Complex Compl1 =  new Complex(1.0, 0.0);
    public static final Complex Compl0 =  new Complex(0.0, 0.0);
    public static final Complex i2pi =  new Complex(0.0, 2.0 * Math.PI);


    /**
      * Calculation of number of the happy tickets
      * through complex numbers.
      * The formula of calculation
      * N =  (1.0 / 2*pi*i) *
      * integral on the closed contour containing number zero,
      * from function
      *    f(z) = ((z^10 - 1) / (z - 1))^6 / z^28
      *  where
      *  10 - basis of a notation,
      *   6 - length of the tickets.
      */
    public static void main (String[] args) {
        double res = 0.0;
        for (int it=0; it<5; it++) {
            long start = System.currentTimeMillis();

            res = test(numberSteps);

            System.out.println("result = "+ res);
            long end = System.currentTimeMillis();
            System.out.println("Total time = "+ (end-start)*0.001);
        }
    }
    

    /**
      * The closed contour is a circle of radius 1
      * with the centre in the beginning of coordinates
      */
/*
    public static double test(int numberSteps1) {
        double st = 2.0 * Math.PI / numberSteps1;
        Complex eps = new Complex(Math.cos(st), Math.sin(st));
        Complex res  = new Complex(0.0, 0.0);
        Complex epsr = new Complex(1.0, 0.0);

        for (int it=0; it<numberSteps1; it++) {
            Complex r1 = eps.mul(epsr);
            Complex delta = r1.sub(epsr);
            res = delta.mul(fi(epsr)).add(res);
            epsr = eps.mul(epsr);
        }
        return res.im() / (2.0*Math.PI);
    }
*/
//--------------------------------------   1 sec - postprocessing...
	public static double test (final int numberSteps1_29)
	{
	  final double st_33 = 2D * java.lang.Math.PI / (double)numberSteps1_29;
	  final double re_34 = java.lang.Math.cos(st_33) /*static*/;
	  final double im_35 = java.lang.Math.sin(st_33) /*static*/;
	  final ORG.netlib.math.complex.Complex res_39 = new ORG.netlib.math.complex.Complex(0D, 0D);
	  //- this is res_39 = new ORG.netlib.math.complex.Complex(0D, 0D);
	  //  res_39.re = 0D;
	  //  res_39.im = 0D;
	  final ORG.netlib.math.complex.Complex epsr_42 = new ORG.netlib.math.complex.Complex(1D, 0D);
	  //- this is epsr_42 = new ORG.netlib.math.complex.Complex(1D, 0D);
	  //  epsr_42.re = 1D;
	  //  epsr_42.im = 0D;
	  ORG.netlib.math.complex.Complex res_273 = res_39;
	  ORG.netlib.math.complex.Complex epsr_272 = epsr_42;
	  int it_271 = 0;
	  while (it_271 < numberSteps1_29) {
	    final double double_278 = epsr_272.re;
	    final double double_280 = epsr_272.im;
	    final double double_283 = epsr_272.im;
	    final double double_285 = epsr_272.re;
	    final double re_294 = re_34 * double_278 - im_35 * double_280 - epsr_272.re;
	    final double im_296 = re_34 * double_283 + im_35 * double_285 - epsr_272.im;
	    final double double_314 = epsr_272.re;
	    final double double_315 = epsr_272.im;
	    final double im_321 = epsr_272.im + 0D * epsr_272.re;
	    final double re_333 = 1D + (double_314 - 0D * double_315);
	    final double re_368 = re_333 - 0D * im_321;
	    final double im_370 = im_321 + 0D * re_333;
	    final double re_383 = re_368 * re_333 - im_370 * im_321;
	    final double im_386 = re_368 * im_321 + im_370 * re_333;
	    final double re_399 = re_383 * re_333 - im_386 * im_321;
	    final double im_402 = re_383 * im_321 + im_386 * re_333;
	    final double re_415 = re_399 * re_333 - im_402 * im_321;
	    final double im_418 = re_399 * im_321 + im_402 * re_333;
	    final double re_431 = re_415 * re_333 - im_418 * im_321;
	    final double im_434 = re_415 * im_321 + im_418 * re_333;
	    final double re_447 = re_431 * re_333 - im_434 * im_321;
	    final double im_450 = re_431 * im_321 + im_434 * re_333;
	    final double re_476 = epsr_272.re - 0D * epsr_272.im;
	    final double im_480 = epsr_272.im + 0D * epsr_272.re;
	    final double re_495 = re_476 * epsr_272.re - im_480 * epsr_272.im;
	    final double im_500 = re_476 * epsr_272.im + im_480 * epsr_272.re;
	    final double re_515 = re_495 * epsr_272.re - im_500 * epsr_272.im;
	    final double im_520 = re_495 * epsr_272.im + im_500 * epsr_272.re;
	    final double re_535 = re_515 * epsr_272.re - im_520 * epsr_272.im;
	    final double im_557 = -(re_515 * epsr_272.im + im_520 * epsr_272.re);
	    final double re_565 = re_447 * re_535 - im_450 * im_557;
	    final double im_568 = re_447 * im_557 + im_450 * re_535;
	    final double re_587 = re_294 * re_565 - im_296 * im_568 + res_273.re;
	    final double im_589 = re_294 * im_568 + im_296 * re_565 + res_273.im;
	    final ORG.netlib.math.complex.Complex res_590 = new ORG.netlib.math.complex.Complex(re_587, im_589);
	    //- this is res_590 = new ORG.netlib.math.complex.Complex(re_587, im_589);
	    //  res_590.re = re_587;
	    //  res_590.im = im_589;
	    final double re_599 = re_34 * epsr_272.re - im_35 * epsr_272.im;
	    final double im_604 = re_34 * epsr_272.im + im_35 * epsr_272.re;
	    final ORG.netlib.math.complex.Complex epsr_605 = new ORG.netlib.math.complex.Complex(re_599, im_604);
	    //- this is epsr_605 = new ORG.netlib.math.complex.Complex(re_599, im_604);
	    //  epsr_605.re = re_599;
	    //  epsr_605.im = im_604;
	    it_271++;
	    epsr_272 = epsr_605;
	    res_273 = res_590;
	    continue;}
	  /*while*/
	  return res_273.im() /*virtual*/ / (2D * java.lang.Math.PI);
	}
//--------------------------------------   2 sec - JScp version 0.0.77


    /** Calculation f(z)
      * @param x Complex number z
      */
    public static Complex fi(Complex z) {
        Complex ff1 = new Complex( 0.0, 0.0);
        Complex ff2 = new Complex( 1.0, 0.0);
        for (int i=0; i<notation; i++) {
            ff1 = ff1.add(ff2);
            ff2 = ff2.mul(z);
        }
        Complex ff3 = degc(ff1, lengthTicket);
        Complex ff4 = ff3.mul(degc(z, c28).conj());
        return ff4;
    }

    /** Calculation of a degree of complex number 
      * @param ccc complex number
      * @param ddd degree
      */
    public static Complex degc(Complex c, int d) {
        Complex r = new Complex( 1.0, 0.0);
        for (int i =  0; i < d; i++) {
            r = r.mul(c);
        }
        return r;
    }

}                
        

         