/* File: Complex.java
 *                             -- A  Java  class  for performing complex
 *                                number arithmetic to double precision.
 *
 * Copyright (c) 1997 - 2001, Alexander Anderson.
 *
 * This  program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published  by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be  useful,  but
 * WITHOUT   ANY   WARRANTY;   without  even  the  implied  warranty  of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR  PURPOSE.   See  the  GNU
 * General Public License for more details.
 *
 * You  should  have  received  a copy of the GNU General Public License
 * along  with  this  program;  if  not,  write  to  the  Free  Software
 * Foundation,  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 *    Alexandr Korlyukov
 *    korlyukov@grsu.grodno.by
 */

import ORG.netlib.math.complex.Complex;

/**
 *  Class for calculation of number of the happy tickets
 *  through complex numbers.
 *  For example,
 *  123 204 - happy ticket, because the sum 1, 2, 3
 *                      is equal to the sum 2, 0, 4
 *  123 254 - is not the happy ticket, because the sum 1, 2, 3
 *                             is not equal to the sum 2, 5, 4
 */
public class HappyTickets{

    public static final int notation  =  2;
    public static final int lengthTicket  =  6;
    public static final int c28  = (notation-1)*(lengthTicket/2)+1;

    public static int numberSteps  =  1000000;

    public static final Complex Compl1 =  new Complex(1.0, 0.0);
    public static final Complex Compl0 =  new Complex(0.0, 0.0);
    public static final Complex i2pi =  new Complex(0.0, 2.0 * Math.PI);

    /**
      * Calculation of number of the happy tickets
      * through complex numbers.
      * The formula of calculation
      * N =  (1.0 / 2*pi*i) *
      * integral on the closed contour containing number zero,
      * from function
      *    f(z) = ((z^10 - 1) / (z - 1))^6 / z^28
      *  where
      *  10 - basis of a notation,
      *   6 - length of the tickets.
      */
    public static void main (String[] args) {
        double res = 0.0;
        for (int it=0; it<5; it++) {
            long start = System.currentTimeMillis();

            res = test(numberSteps);

            System.out.println("result = "+ res);
            long end = System.currentTimeMillis();
            System.out.println("Total time = "+ (end-start)*0.001);
        }
    }
    

    /**
      * The closed contour is a circle of radius 1
      * with the centre in the beginning of coordinates
      */
/*
    public static double test(int numberSteps1) {
        double st = 2.0 * Math.PI / numberSteps1;
        Complex eps = new Complex(Math.cos(st), Math.sin(st));
        Complex res  = new Complex(0.0, 0.0);
        Complex epsr = new Complex(1.0, 0.0);

        for (int it=0; it<numberSteps1; it++) {
            Complex r1 = eps.mul(epsr);
            Complex delta = r1.sub(epsr);
            res = delta.mul(fi(epsr)).add(res);
            epsr = eps.mul(epsr);
        }
        return res.im() / (2.0*Math.PI);
    }
*/
//--------------------------------------  11 sec - postprocessing...
	public static double test (final int numberSteps1_29)
	{
	  final double st_33 = 2D * java.lang.Math.PI / (double)numberSteps1_29;
	  final double re_34 = java.lang.Math.cos(st_33) /*static*/;
	  final double im_35 = java.lang.Math.sin(st_33) /*static*/;
	  final ORG.netlib.math.complex.Complex res_39 = new ORG.netlib.math.complex.Complex(0D, 0D);
	  //- this is res_39 = new ORG.netlib.math.complex.Complex(0D, 0D);
	  //  res_39.re = 0D;
	  //  res_39.im = 0D;
	  final ORG.netlib.math.complex.Complex epsr_42 = new ORG.netlib.math.complex.Complex(1D, 0D);
	  //- this is epsr_42 = new ORG.netlib.math.complex.Complex(1D, 0D);
	  //  epsr_42.re = 1D;
	  //  epsr_42.im = 0D;
	  ORG.netlib.math.complex.Complex res_2705 = res_39;
	  ORG.netlib.math.complex.Complex epsr_2704 = epsr_42;
	  int it_2703 = 0;
	  while (it_2703 < numberSteps1_29) {
	    final double A_0_0_2727 = epsr_2704.re;
	    final double A_0_1_2729 = epsr_2704.im;
	    final double double_2731 = epsr_2704.im;
	    final double A_1_1_2734 = epsr_2704.re;
	    final double A_0_0_2916 = re_34 * A_0_0_2727 + im_35 * -double_2731 - epsr_2704.re;
	    final double A_0_1_2921 = re_34 * A_0_1_2729 + im_35 * A_1_1_2734 - epsr_2704.im;
	    final double A_0_0_3057 = epsr_2704.re;
	    final double A_0_1_3059 = epsr_2704.im;
	    final double double_3061 = epsr_2704.im;
	    final double s_3155 = A_0_1_3059 + 0D * epsr_2704.re;
	    final double A_0_0_3243 = 1D + (A_0_0_3057 + 0D * -double_3061);
	    final double s_3520 = A_0_0_3243 + 0D * -s_3155;
	    final double s_3580 = s_3155 + 0D * A_0_0_3243;
	    final double s_3688 = s_3520 * A_0_0_3243 + s_3580 * -s_3155;
	    final double s_3750 = s_3520 * s_3155 + s_3580 * A_0_0_3243;
	    final double s_3859 = s_3688 * A_0_0_3243 + s_3750 * -s_3155;
	    final double s_3921 = s_3688 * s_3155 + s_3750 * A_0_0_3243;
	    final double s_4030 = s_3859 * A_0_0_3243 + s_3921 * -s_3155;
	    final double s_4092 = s_3859 * s_3155 + s_3921 * A_0_0_3243;
	    final double s_4201 = s_4030 * A_0_0_3243 + s_4092 * -s_3155;
	    final double s_4263 = s_4030 * s_3155 + s_4092 * A_0_0_3243;
	    final double s_4372 = s_4201 * A_0_0_3243 + s_4263 * -s_3155;
	    final double s_4434 = s_4201 * s_3155 + s_4263 * A_0_0_3243;
	    final double A_0_0_4519 = epsr_2704.re;
	    final double A_0_1_4521 = epsr_2704.im;
	    final double s_4557 = A_0_0_4519 + 0D * -epsr_2704.im;
	    final double s_4617 = A_0_1_4521 + 0D * epsr_2704.re;
	    final double A_0_0_4690 = epsr_2704.re;
	    final double A_0_1_4692 = epsr_2704.im;
	    final double s_4729 = s_4557 * A_0_0_4690 + s_4617 * -epsr_2704.im;
	    final double s_4791 = s_4557 * A_0_1_4692 + s_4617 * epsr_2704.re;
	    final double A_0_0_4865 = epsr_2704.re;
	    final double A_0_1_4867 = epsr_2704.im;
	    final double s_4904 = s_4729 * A_0_0_4865 + s_4791 * -epsr_2704.im;
	    final double s_4966 = s_4729 * A_0_1_4867 + s_4791 * epsr_2704.re;
	    final double A_0_0_5040 = epsr_2704.re;
	    final double A_0_1_5042 = epsr_2704.im;
	    final double s_5079 = s_4904 * A_0_0_5040 + s_4966 * -epsr_2704.im;
	    final double im_5204 = -(s_4904 * A_0_1_5042 + s_4966 * epsr_2704.re);
	    final double s_5262 = s_4372 * s_5079 + s_4434 * -im_5204;
	    final double s_5324 = s_4372 * im_5204 + s_4434 * s_5079;
	    final double A_0_0_5579 = A_0_0_2916 * s_5262 + A_0_1_2921 * -s_5324 + res_2705.re;
	    final double A_0_1_5584 = A_0_0_2916 * s_5324 + A_0_1_2921 * s_5262 + res_2705.im;
	    final ORG.netlib.math.complex.Complex res_5620 = new ORG.netlib.math.complex.Complex(A_0_0_5579, A_0_1_5584);
	    //- this is res_5620 = new ORG.netlib.math.complex.Complex(A_0_0_5579, A_0_1_5584);
	    //  res_5620.re = A_0_0_5579;
	    //  res_5620.im = A_0_1_5584;
	    final double A_0_0_5642 = epsr_2704.re;
	    final double A_0_1_5644 = epsr_2704.im;
	    final double s_5681 = re_34 * A_0_0_5642 + im_35 * -epsr_2704.im;
	    final double s_5743 = re_34 * A_0_1_5644 + im_35 * epsr_2704.re;
	    final ORG.netlib.math.complex.Complex epsr_5790 = new ORG.netlib.math.complex.Complex(s_5681, s_5743);
	    //- this is epsr_5790 = new ORG.netlib.math.complex.Complex(s_5681, s_5743);
	    //  epsr_5790.re = s_5681;
	    //  epsr_5790.im = s_5743;
	    it_2703++;
	    epsr_2704 = epsr_5790;
	    res_2705 = res_5620;
	    continue;}
	  /*while*/
	  return res_2705.im() /*virtual*/ / (2D * java.lang.Math.PI);
	}
//--------------------------------------  14 sec - JScp version 0.0.77


    /** Calculation f(z)
      * @param x Complex number z
      */
    public static Complex fi(Complex z) {
        Complex ff1 = new Complex( 0.0, 0.0);
        Complex ff2 = new Complex( 1.0, 0.0);
        for (int i=0; i<notation; i++) {
            ff1 = ff1.add(ff2);
            ff2 = ff2.mul(z);
        }
        Complex ff3 = degc(ff1, lengthTicket);
        Complex ff4 = ff3.mul(degc(z, c28).conj());
        return ff4;
    }

    /** Calculation of a degree of complex number 
      * @param ccc complex number
      * @param ddd degree
      */
    public static Complex degc(Complex c, int d) {
        Complex r = new Complex( 1.0, 0.0);
        for (int i =  0; i < d; i++) {
            r = r.mul(c);
        }
        return r;
    }

}                
        

         